/*
 * Project name:
      THERMO click - Thermocouple Test
 * Copyright:
      (c) mikroElektronika, 2012.
 * Revision History:
      20120709:
      - Initial release (DO);
 * Description:
      This is a simple project which demonstrates the use of THERMO click board.
      Temperature measured by the thermocouple is converter by MAX31855 and displayed on LCD.
      Displayed temperature is in degree Celsius.
      Example also includes fault detection. (Open circuit, Short to GND and Short to VCC).
 * Test configuration:
     MCU:             PIC18F45K22
                      http://ww1.microchip.com/downloads/en/DeviceDoc/41412F.pdf
     Dev. Board:      EasyPIC v7
                      http://www.mikroe.com/eng/products/view/757/easypic-v7-development-system/
     Oscillator:      HS-PLL, 8.000 MHz Crystal, 32.000 MHz MCU Clock
     Ext. Modules:    THERMO click - ac:THERMO_click
                      http://www.mikroe.com/eng/products/view/930/thermo-click/
     SW:              mikroC PRO for PIC
                      http://www.mikroe.com/eng/products/view/7/mikroc-pro-for-pic/
 * NOTES:
      - Place THERMO Click board in the mikroBUS socket 1.
      - Place LCD and turn on LCD backlight switch SW4.6.
      - Set power supply jumper(J16) in 3.3V position.
*/
// Lcd module connections
sbit LCD_RS at LATB4_bit;
sbit LCD_EN at LATB5_bit;
sbit LCD_D4 at LATB0_bit;
sbit LCD_D5 at LATB1_bit;
sbit LCD_D6 at LATB2_bit;
sbit LCD_D7 at LATB3_bit;

sbit LCD_RS_Direction at TRISB4_bit;
sbit LCD_EN_Direction at TRISB5_bit;
sbit LCD_D4_Direction at TRISB0_bit;
sbit LCD_D5_Direction at TRISB1_bit;
sbit LCD_D6_Direction at TRISB2_bit;
sbit LCD_D7_Direction at TRISB3_bit;
// End Lcd module connections

sbit THERMO_CS     at LATE0_bit;
sbit THERMO_CS_Dir at TRISE0_bit;

int tmp, intTemp, remTemp;
float temperature;
char s_temp[16];
short out_size = 0;
short address, buffer;
unsigned short temp_byte[4] = {0, 0, 0, 0};

void MAX31855_Read() {
  THERMO_CS = 0;
  Delay_ms(350);
  temp_byte[0] = SPI_Read(0);
  temp_byte[1] = SPI_Read(0);
  temp_byte[2] = SPI_Read(0);
  temp_byte[3] = SPI_Read(0);
  THERMO_CS = 1;
}

void Display_Temp_Value(void) {
  tmp = temp_byte[0];
  tmp = tmp << 8;
  tmp = tmp | temp_byte[1];
  remTemp = tmp >> 2;
  remTemp = remTemp & 0x03;                     // Decimal part of temperature value
  temperature = remTemp * 0.25;
  intTemp = tmp >> 4;                           // Intiger part of temperature value
  temperature += intTemp;                       // Temperature value
  Lcd_Out(1,1,"Temperature:");                  // Write text in first row
  sprintf(s_temp,"%4.2f C",temperature);
  if (out_size > strlen(s_temp)){
    Lcd_Out(2, 1, "                ");
  }
  out_size = strlen(s_temp);
  Lcd_Out(2,1,s_temp);
}

void main() {
  PLLEN_bit = 1;                     // Enable PLL
  Delay_ms(150);

  CM1CON = 0;                        // Turn off comparators
  CM2CON = 0;
  ANSELB = 0;                        // Configure PORTB pins as digital
  ANSELC = 0;                        // Configure PORTC pins as digital

  THERMO_CS_Dir = 0;
  THERMO_CS = 1;

  Lcd_Init();                        // Initialize Lcd

  Lcd_Cmd(_LCD_CLEAR);               // Clear display
  Lcd_Cmd(_LCD_CURSOR_OFF);          // Cursor off
  Lcd_Out(1,1,"Thermo test");        // Write text in first row
  Lcd_Out(2,1,"starting...");        // Write text in second row
  Delay_ms(2000);

  SPI1_Init_Advanced(_SPI_MASTER_OSC_DIV16, _SPI_DATA_SAMPLE_MIDDLE, _SPI_CLK_IDLE_LOW, _SPI_LOW_2_HIGH);  // Initialize SPI communication
  SPI_Set_Active(&SPI1_Read, &SPI1_Write);      // Sets the SPI1 module active

  Lcd_Cmd(_LCD_CLEAR);
  Lcd_Out(1,1,"Reading");                       // Write text in first row
  Lcd_Out(2,1,"temperature ...");               // Write text in second row
  Delay_ms(2000);

  Lcd_Cmd(_LCD_CLEAR);

  while(1) {
    MAX31855_Read();
    //
    if((temp_byte[1] & 0x01) == 0x01){          // Fault detection
      Lcd_Cmd(_LCD_CLEAR);
      Lcd_Out(2,1,"Error!");
      if((temp_byte[3] & 0x01) == 0x01){        // Open circuit fault?
         Lcd_Out(1,1,"Open circuit");           // Write text in first row
         Delay_ms(1000);
      }
      //
      if((temp_byte[3] & 0x02) == 0x02){        // Short to GND fault?
         Lcd_Out(1,1,"Short to GND");           // Write text in first row
         Delay_ms(1000);
      }
      //
      if((temp_byte[3] & 0x04) == 0x04){        // Short to Vcc fault?
         Lcd_Out(1,1,"Short to Vcc");           // Write text in first row
         Delay_ms(1000);
      }
    }
    else{
       Display_Temp_Value();
    }
    Delay_ms(500);
  }
}